-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    ecu.vhd
-- Description:  Engine Controll Unit - composite from QUEUE, ECU_FSM and MOVE
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_signed.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity ECU_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16;
      QUEUE_SIZE     : integer := 15        
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;
      -- ----------------- FIFO INPUT --------------------------------
      X_IN           : in std_logic_vector(DATA_WIDTH - 1 downto 0);
      Y_IN           : in std_logic_vector(DATA_WIDTH - 1 downto 0);
      
      ACT_IN         : in std_logic;
      VLD_IN         : out std_logic;
      -- ----------------- MOVE OUTPUT -------------------------
      X_SPEED        : out std_logic_vector(DATA_WIDTH - 1 downto 0);
      Y_SPEED        : out std_logic_vector(DATA_WIDTH - 1 downto 0)
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture ECU_ARCH of ECU_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- signals 
signal sig_act       : std_logic;
signal sig_vld       : std_logic;
signal sig_l         : std_logic;
signal sig_r         : std_logic;
signal sig_t         : std_logic;
signal sig_b         : std_logic;

signal sig_act_fifo  : std_logic;
signal sig_vld_fifo  : std_logic;
signal sig_x         : std_logic_vector(DATA_WIDTH - 1 downto 0);
signal sig_y         : std_logic_vector(DATA_WIDTH - 1 downto 0);

begin
   queue : entity work.QUEUE_ENT
   generic map(
      DATA_WIDTH     => DATA_WIDTH,
      QUEUE_SIZE     => QUEUE_SIZE     
   )
   
   port map(
      CLK            => CLK,
      RST            => RST,
      -- ----------------- INPUT INTERFACE --------------------------------
      X_IN           => X_IN,
      Y_IN           => Y_IN,
      VLD_IN         => VLD_IN,
      ACT_IN         => ACT_IN,
      -- ----------------- OUTPUT INTERFACE -------------------------
      X_OUT          => sig_x,
      Y_OUT          => sig_y,
      VLD_OUT        => sig_vld_fifo,
      ACT_OUT        => sig_act_fifo 
   );

   ecu_fsm : entity work.ECU_FSM_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )  
   port map(
      CLK            => CLK,
      RST            => RST,
      -- ----------------- FIFO INPUT --------------------------------
      X              => sig_x,
      Y              => sig_y,
      
      FIFO_ACT       => sig_act_fifo,
      FIFO_VLD       => sig_vld_fifo,
      -- ----------------- MOVE OUTPUT -------------------------
      L              => sig_l,
      R              => sig_r,
      T              => sig_t,
      B              => sig_b,
      
      ACT            => sig_act,
      VLD            => sig_vld
   );
   
   move : entity work.MOVE_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )     
   port map(
      CLK            => CLK,
      RST            => RST,
      
      ACT            => sig_act,
      VLD            => sig_vld,

      -- ----------------- DIRECION INPUT -------------------------
      L              => sig_l,
      R              => sig_r,
      T              => sig_t,
      B              => sig_b,
      -- ----------------- SPEED OUTPUT ---------------------------
      X_SPEED        => X_SPEED,
      Y_SPEED        => Y_SPEED
   ); 
   
end architecture;