-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    direction.vhd
-- Description:  Transform x and y to left, right, top or bottom direct
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity DIR_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16     
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;
      WE             : in std_logic; -- enable writing to output registers
      -- ----------------- DIRECION OUTPUT ------------------------
      L              : out std_logic;
      R              : out std_logic;
      T              : out std_logic;
      B              : out std_logic;
      -- ----------------- X AND Y INPUT --------------------------
      X              : in std_logic_vector(DATA_WIDTH - 1 downto 0);
      Y              : in std_logic_vector(DATA_WIDTH - 1 downto 0)
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture DIR_ARCH of DIR_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================

-- data  signals
signal sig_x_old           : std_logic_vector(DATA_WIDTH - 1 downto 0);
signal sig_y_old           : std_logic_vector(DATA_WIDTH - 1 downto 0);
signal sig_l               : std_logic;
signal sig_r               : std_logic;
signal sig_t               : std_logic;
signal sig_b               : std_logic;


begin
   --compare actual position with previous position end evaluate direct
   cmp : process (CLK, RST, X, Y, sig_x_old, sig_y_old)
   begin
      if (RST = '1') then
         sig_l <= '0';
         sig_r <= '0';
         sig_t <= '0';
         sig_b <= '0';
      elsif (rising_edge( CLK )) then
         if (X < sig_x_old) then
            sig_l <= '1';
            sig_r <= '0';
            sig_t <= '0';
            sig_b <= '0';
         elsif (X > sig_x_old) then
            sig_l <= '0';
            sig_r <= '1';
            sig_t <= '0';
            sig_b <= '0';
         elsif (Y < sig_y_old) then
            sig_l <= '0';
            sig_r <= '0';
            sig_t <= '0';
            sig_b <= '1';
         elsif (Y > sig_y_old) then
            sig_l <= '0';
            sig_r <= '0';
            sig_t <= '1';
            sig_b <= '0';
         else
            sig_l <= '0';
            sig_r <= '0';
            sig_t <= '0';
            sig_b <= '0';
         end if; 
      end if; 
   end process;
   
   -- get values to output
   output_registers : process (WE, CLK, RST)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            L <= '0';
            R <= '0';
            T <= '0';
            B <= '0';
         elsif (WE = '1') then
            L <= sig_l;
            R <= sig_r;
            T <= sig_t;
            B <= sig_b;
         end if;
      end if;
   end process;
   
   -- save previous value to registers
   imput_registers : process (CLK, RST)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            sig_x_old <= (others => '0');
            sig_y_old <= (others => '0');
         else
            sig_x_old <= X;
            sig_y_old <= Y;
         end if;
      end if;
   end process;
end architecture;
