library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 20 ns;
  constant reset_time   : time := 100 ns;
  constant data_width   : integer := 16;
  
  -- prepajacie signaly
  signal clk         : std_logic;
  signal rst         : std_logic;  
  signal enr         : std_logic;
  signal enw         : std_logic;
  signal dataout_x   : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal dataout_y   : std_logic_vector(DATA_WIDTH-1 downto 0);
  signal datain      : std_logic_vector (2*DATA_WIDTH-1 downto 0);
  signal empty       : std_logic;
  signal err         : std_logic;
  signal full        : std_logic;
      

begin
  -- instancia testovanej komponenty
  UUT : entity work.fifo_ent
  generic map (
            SIZE        => 14,
            DATA_WIDTH  => DATA_WIDTH
  )
  port map( CLK         => clk,
            RST         => rst,
            READ_EN     => enr,
            WRITE_EN    => enw,
            DATA_OUT(31 downto 16) => dataout_x,
            DATA_OUT(15 downto 0)  => dataout_y,
            DATA_IN     => datain,
            EMPTY       => empty,
            ERROR       => err,
            FULL        => full
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clk     <='1';
    wait for clk_period/2;
    clk     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      rst   <= '1';
      wait for reset_time;
      rst   <= '0';
      --wait for 10*reset_time;
      --rst   <= '1';
      wait;
   end process;

  -- generovanie vstupov
  behaviour : process
  begin
    enr <= '0';
    enw <= '0';
    datain <= std_logic_vector(to_unsigned(0, 2*DATA_WIDTH));
    wait for reset_time;
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    datain <= std_logic_vector(to_unsigned(3, 2*DATA_WIDTH));
    enw <= '1';
    --enr <= '1';
    wait for clk_period;
    enw <= '0';
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    
    --enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
   

    enw <= '1';
    datain <= std_logic_vector(to_unsigned(767, 2*DATA_WIDTH));
    wait for clk_period;
    enw <= '0';
    
    wait for reset_time;
    enr <= '1';
    wait for clk_period;
    enr <= '0';
    wait for reset_time;
    wait;
  end process behaviour; 
   
end architecture tb_arch;
