library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 20 ns;
  constant reset_time   : time := 100 ns;
  constant data_width   : integer := 16;
  
  -- prepajacie signaly
  signal clk            : std_logic;
  signal rst            : std_logic;  
      -- ----------------- FIFO INTERFACE --------------------------------
  signal data_in        : std_logic_vector(2*data_width - 1 downto 0);
  signal write_en       : std_logic;
  signal full           : std_logic;
  signal error          : std_logic;
      -- ----------------- OUTPUT INTERFACE -------------------------      
  signal x              : std_logic_vector(data_width - 1 downto 0);
  signal y              : std_logic_vector(data_width - 1 downto 0);
  signal act            : std_logic;
  signal vld            : std_logic;  
      

begin
  -- instancia testovanej komponenty
  UUT : entity work.FIFO_IN_ENT
  port map( CLK            => clk,
            RST            => rst,
            -- ----------------- FIFO INTERFACE --------------------------------
            DATA_IN        => data_in,
            WRITE_EN       => write_en,
            FULL           => full,
            ERROR          => error, 
            -- ----------------- OUTPUT INTERFACE -------------------------      
            X              => x,
            Y              => y,
            ACT            => act, 
            VLD            => vld
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clk     <='1';
    wait for clk_period/2;
    clk     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      rst   <= '1';
      wait for reset_time;
      rst   <= '0';
      --wait for 10*reset_time;
      --rst   <= '1';
      wait;
   end process;

  -- generovanie vstupov
  behaviour : process
  begin
    act        <= '0';
    full      <= '1';
    x   <= std_logic_vector(to_unsigned(1, DATA_WIDTH));
    y   <= std_logic_vector(to_unsigned(6, DATA_WIDTH));
    wait for reset_time;
    wait for clk_period;
    act        <= '1';
    wait for reset_time;
    full      <= '0';
    wait for 3*clk_period;
    act        <= '0';
    
    wait;
  end process behaviour; 
   
end architecture tb_arch;
