-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    fifo_out.vhd
-- Description:  Controll FIFO reading interface by ACT and VLD
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_signed.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity FIFO_OUT_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16     
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;
      -- ----------------- FIFO INTERFACE --------------------------------
      DATA_OUT       : in std_logic_vector(2*DATA_WIDTH - 1 downto 0);
      READ_EN        : out std_logic;
      EMPTY          : in std_logic;
      ERROR          : in std_logic;
      -- ----------------- OUTPUT INTERFACE -------------------------      
      X              : out std_logic_vector(DATA_WIDTH - 1 downto 0);
      Y              : out std_logic_vector(DATA_WIDTH - 1 downto 0);
      ACT            : in std_logic;
      VLD            : out std_logic         
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture FIFO_OUT_ARCH of FIFO_OUT_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state, read_state); 

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next: state_type;

-- signals



begin
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic: process( state_reg, EMPTY, ACT) --stavova logika
   begin
      case state_reg is
         when init_state =>
            if (ACT = '1' and EMPTY = '0') then
               READ_EN     <= '1';
               VLD         <= '0';
               state_next  <= read_state;
            else
               READ_EN     <= '0';
               VLD         <= '0';
               state_next  <= init_state;
            end if;
            
         when read_state =>
            if (ACT = '1') then
               READ_EN     <= '0';
               VLD         <= '1';
               state_next  <= read_state;
            else
               READ_EN     <= '0';
               VLD         <= '0';
               state_next  <= init_state;
            end if;
    
         when others => 
            READ_EN     <= '0';
            VLD         <= '0';
            state_next  <= init_state;
      end case;
   end process;  

   -- transform fifo value to x and y
   X <= DATA_OUT(2*DATA_WIDTH-1 downto DATA_WIDTH);
   Y <= DATA_OUT(DATA_WIDTH-1 downto 0);
end architecture;