-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    queue.vhd
-- Description:  Queue is realized as FIFO with input and output interface
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_signed.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity QUEUE_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16;
      QUEUE_SIZE     : integer := 15      
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;
      -- ----------------- INPUT INTERFACE --------------------------------
      X_IN           : in std_logic_vector(DATA_WIDTH - 1 downto 0);
      Y_IN           : in std_logic_vector(DATA_WIDTH - 1 downto 0);
      VLD_IN         : out std_logic;
      ACT_IN         : in std_logic;
      -- ----------------- OUTPUT INTERFACE -------------------------
      X_OUT          : out std_logic_vector(DATA_WIDTH - 1 downto 0);
      Y_OUT          : out std_logic_vector(DATA_WIDTH - 1 downto 0);
      VLD_OUT        : out std_logic;
      ACT_OUT        : in std_logic
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture QUEUE_ARCH of QUEUE_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- signals 
signal sig_data_in       : std_logic_vector(2*DATA_WIDTH - 1 downto 0);
signal sig_data_out      : std_logic_vector(2*DATA_WIDTH - 1 downto 0);

signal sig_write_en      : std_logic;
signal sig_full          : std_logic;
signal sig_read_en       : std_logic;
signal sig_empty         : std_logic;
signal sig_error         : std_logic;

begin
   -- input interface
   fifo_in : entity work.FIFO_IN_ENT
   generic map(
      DATA_WIDTH    => DATA_WIDTH
   )    
   port map(
      CLK            => CLK,
      RST            => RST,
      -- ----------------- FIFO INTERFACE --------------------------------
      DATA_IN        => sig_data_in,
      WRITE_EN       => sig_write_en,
      FULL           => sig_full,
      ERROR          => sig_error ,
      -- ----------------- OUTPUT INTERFACE -------------------------      
      X              => X_IN,
      Y              => Y_IN,
      ACT            => ACT_IN,
      VLD            => VLD_IN        
   );
   
   -- fifo
   fifo : entity work.FIFO_ENT
   generic map(
      SIZE           => QUEUE_SIZE,
      DATA_WIDTH     => DATA_WIDTH
   )    
   port map(
      CLK            => CLK,
      RST            => RST,
      -- ----------------- FIFO INTERFACE --------------------------------
      DATA_IN        => sig_data_in,
      DATA_OUT       => sig_data_out,
      READ_EN        => sig_read_en,
      WRITE_EN       => sig_write_en,
      EMPTY          => sig_empty, 
      FULL           => sig_full,
      ERROR          => sig_error 
   );
   
   -- output interface
   fifo_out : entity work.FIFO_OUT_ENT
   generic map(
      DATA_WIDTH    => DATA_WIDTH
   )   
   port map(
      CLK            => CLK,
      RST            => RST,
      -- ----------------- FIFO INTERFACE --------------------------------
      DATA_OUT       => sig_data_out,
      READ_EN        => sig_read_en,
      EMPTY          => sig_empty,
      ERROR          => sig_error,
      -- ----------------- OUTPUT INTERFACE -------------------------      
      X              => X_OUT,
      Y              => Y_OUT,
      ACT            => ACT_OUT,
      VLD            => VLD_OUT        
   );
   
end architecture;