-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    mem_topl_level.vhd
-- Description:  Map Unit + Map Memory Unit for testing memmory subsystem
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         15.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity MEM_TOP_LEVEL_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16;  
      ADDR_WIDTH  : integer := 13       
   );
   
   port
   (
      -- ---------------- INPUT INTERFACE -------------------
      CLK               : in  std_logic;
      RST               : in  std_logic;  
      ACT               : in  std_logic;
      VLD               : out std_logic;   
      X                 : in std_logic_vector(DATA_WIDTH-1 downto 0);
      Y                 : in std_logic_vector(DATA_WIDTH-1 downto 0);
      BARRIER           : in std_logic_vector(3 downto 0);
      
      
      -- ----------------- PORT B ---------------------------       
      ADDR_X_B          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      ADDR_Y_B          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      DATA_OUT_B        : out std_logic_vector(3 downto 0)
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture MEM_TOP_LEVEL_ARCH of MEM_TOP_LEVEL_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- data signals
                
-- control signals       
signal sig_addr_x_a          : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_addr_y_a          : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_rd_wr_a           : std_logic;
signal sig_data_out_a        : std_logic_vector(3 downto 0);
signal sig_data_in_a         : std_logic_vector(3 downto 0);
signal sig_act_a             : std_logic;
signal sig_vld_a             : std_logic;
 
begin

   map_unit : entity work.MU_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      CLK            => CLK,
      RST            => RST, 
      ACT            => ACT,
      -- ----------------- INPUT/OUTPUT INTERFACE ----------------------------------
      X              => X, 
      Y              => Y,
      BARRIER        => BARRIER,
      VLD            => VLD,
      -- ----------------- MEMORY INTERFACE ---------------------------------   
      ADDR_X         => sig_addr_x_a,
      ADDR_Y         => sig_addr_y_a,
      RD_WR          => sig_rd_wr_a,
      DATA_OUT       => sig_data_in_a,
      DATA_IN        => sig_data_out_a,
      MEM_ACT        => sig_act_a,
      MEM_VLD        => sig_vld_a
   );
   
   map_memory_unit : entity work.MMU_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH,
      ADDR_WIDTH  => ADDR_WIDTH
   )   
   port map(
      -- ----------------- PORT A ---------------------------
      CLK_A             => CLK,
      RST_A             => RST,       
      ADDR_X_A          => sig_addr_x_a,
      ADDR_Y_A          => sig_addr_y_a,
      RD_WR_A           => sig_rd_wr_a,
      DATA_OUT_A        => sig_data_out_a,
      DATA_IN_A         => sig_data_in_a,
      ACT_A             => sig_act_a,
      VLD_A             => sig_vld_a,
      
      -- ----------------- PORT B ---------------------------
      CLK_B             => CLK,
      RST_B             => RST,        
      ADDR_X_B          => ADDR_X_B,
      ADDR_Y_B          => ADDR_Y_B,
      RD_WR_B           => '0',
      DATA_OUT_B        => DATA_OUT_B,
      DATA_IN_B         => "0000",
      ACT_B             => '1',
      VLD_B             => open
   );
  
end architecture;
