library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 20 ns;
  constant reset_time   : time := 100 ns;
  constant data_width   : integer := 16;
  
  -- prepajacie signaly
  -- ---------------- INPUT INTERFACE -------------------
  signal clk               : std_logic;
  signal rst               : std_logic;  
  signal act               : std_logic;
  signal vld               : std_logic;   
  signal x                 : std_logic_vector(data_width-1 downto 0);
  signal y                 : std_logic_vector(data_width-1 downto 0);
  signal barrier           : std_logic_vector(3 downto 0);
      
      
  -- ----------------- port b ---------------------------       
  signal addr_x_b          : std_logic_vector(data_width-1 downto 0);
  signal addr_y_b          : std_logic_vector(data_width-1 downto 0);
  signal data_out_b        : std_logic_vector(3 downto 0);
  
      

begin
  -- instancia testovanej komponenty
  UUT : entity work.MEM_TOP_LEVEL_ENT
  port map( CLK               => clk,
            RST               => rst,  
            ACT               => act, 
            VLD               => vld,   
            X                 => x,
            Y                 => y,
            BARRIER           => barrier,      
            ADDR_X_B          => addr_x_b,
            ADDR_Y_B          => addr_y_b,
            DATA_OUT_B        => data_out_b 
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clk     <='1';
    wait for clk_period/2;
    clk     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      rst   <= '1';
      wait for reset_time;
      rst   <= '0';
      --wait for 10*reset_time;
      --rst   <= '1';
      wait;
   end process;

  -- generovanie vstupov
  porta : process
  begin
    x     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    y     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
      
    act          <= '0';
    barrier      <= "0101";
    
    wait for 2*reset_time;
    act          <= '1';
    wait for 7*reset_time;
    act          <= '0';
    wait for reset_time;
    act          <= '1';
    wait for 7*reset_time;
    act          <= '0';
    wait;
  end process porta; 
  
  portb : process
  begin
    addr_x_b     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    addr_y_b     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    wait for 18*reset_time;
    addr_x_b     <= std_logic_vector(to_unsigned(22, DATA_WIDTH));
    addr_y_b     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    wait for 2*reset_time;
    addr_x_b     <= std_logic_vector(to_unsigned(24, DATA_WIDTH));
    addr_y_b     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    wait for 2*reset_time;
    addr_x_b     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
	 wait for clk_period;
    addr_y_b     <= std_logic_vector(to_unsigned(22, DATA_WIDTH));
    wait for 2*reset_time;
    addr_x_b     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    addr_y_b     <= std_logic_vector(to_unsigned(24, DATA_WIDTH));
    wait;
  end process portb; 
   
end architecture tb_arch;
