library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 20 ns;
  constant reset_time   : time := 100 ns;
  constant data_width   : integer := 16;
  
  -- prepajacie signaly
  signal clk_a             : std_logic;
  signal rst_a             : std_logic;        
  signal addr_x_a          : std_logic_vector(data_width-1 downto 0);
  signal addr_y_a          : std_logic_vector(data_width-1 downto 0);
  signal rd_wr_a           : std_logic;
  signal data_out_a        : std_logic_vector(3 downto 0);
  signal data_in_a         : std_logic_vector(3 downto 0);
  signal act_a             : std_logic;
  signal vld_a             : std_logic;
      
  signal clk_b             : std_logic;
  signal rst_b             : std_logic;        
  signal addr_x_b          : std_logic_vector(data_width-1 downto 0);
  signal addr_y_b          : std_logic_vector(data_width-1 downto 0);
  signal rd_wr_b           : std_logic;
  signal data_out_b        : std_logic_vector(3 downto 0);
  signal data_in_b         : std_logic_vector(3 downto 0);
  signal act_b             : std_logic;
  signal vld_b             : std_logic;
  
      

begin
  -- instancia testovanej komponenty
  UUT : entity work.MMU_ENT
  port map( CLK_A             => clk_a,
            RST_A             => rst_a,        
            ADDR_X_A          => addr_x_a, 
            ADDR_Y_A          => addr_y_a,
            RD_WR_A           => rd_wr_a,
            DATA_OUT_A        => data_out_a,
            DATA_IN_A         => data_in_a,
            ACT_A             => act_a,
            VLD_A             => vld_a,
            
            -- ----------------- PORT B ---------------------------
            CLK_B             => clk_b,
            RST_B             => rst_b,        
            ADDR_X_B          => addr_x_b,
            ADDR_Y_B          => addr_y_b,
            RD_WR_B           => rd_wr_b,
            DATA_OUT_B        => data_out_b,
            DATA_IN_B         => data_in_b,
            ACT_B             => act_b,
            VLD_B             => vld_b
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clk_a     <='1';
    clk_b     <='1';
    wait for clk_period/2;
    clk_a     <='0';
    clk_b     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      rst_a   <= '1';
      rst_b   <= '1';
      wait for reset_time;
      rst_a   <= '0';
      rst_b   <= '0';
      --wait for 10*reset_time;
      --rst   <= '1';
      wait;
   end process;

  -- generovanie vstupov
  porta : process
  begin
    addr_x_a     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    addr_y_a     <= std_logic_vector(to_unsigned(1, DATA_WIDTH));
      
    act_a        <= '0';
    rd_wr_a      <= '0';
    
    wait for 2*reset_time;
    data_in_a    <= "1111";
    rd_wr_a      <= '1';
    act_a        <= '1';
    wait for 2*clk_period;
    act_a        <= '0';
    
    
    --wait for clk_period;
    --rd_wr      <= '1';
    --act        <= '1';
    --do         <= "0101";
    --wait for 2*clk_period;
    --act        <= '0';
    wait;
  end process porta; 
  
  portb : process
  begin
    addr_x_b     <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    addr_y_b     <= std_logic_vector(to_unsigned(1, DATA_WIDTH));
    data_in_b    <= "1001";  
  
    act_b        <= '0';
    rd_wr_b      <= '0';
    --rd_wr_b      <= '1';
    wait for 2*reset_time;
    --wait for 2*clk_period;
    act_b        <= '1';
    wait for 4*clk_period;
    act_b        <= '0';
    wait;
  end process portb; 
   
end architecture tb_arch;
