-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    mu_fsm.vhd
-- Description:  Controll MU, get data from BUS, detect data change and write new data to memmory
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.1.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
--use ieee.numeric_std.all;
--use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity MU_FSM_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic; 
      
      -- ----------------- INPUT INTERFACE ----------------------- 
      WB_X           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      WB_Y           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      WB_BAR         : in std_logic_vector(DATA_WIDTH-1 downto 0);
      WB_ACT         : out  std_logic;
      WB_VLD         : in  std_logic; 
	  RDY            : in  std_logic; 
              
      -- ----------------- OUTPUT INTERFACE ----------------------
      MU_X           : out std_logic_vector(DATA_WIDTH-1 downto 0);
      MU_Y           : out std_logic_vector(DATA_WIDTH-1 downto 0);
      MU_BAR         : out std_logic_vector(3 downto 0);
      MU_ACT         : out  std_logic;
      MU_VLD         : in  std_logic
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture MU_FSM_ARCH of MU_FSM_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state, wait_state, get_value_state, save_state); --stavy FSM

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next: state_type;

-- data signals
signal reg_X                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_Y                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_BAR                : std_logic_vector(DATA_WIDTH-1 downto 0);

-- control signals
signal sig_act                : std_logic;
signal sig_mu_vld             : std_logic;

signal sig_change_val         : std_logic;

begin

   sig_mu_vld <= MU_VLD;
   -- detect event on input data => enable evaluating
   change_register : process (CLK, reg_X, reg_Y, WB_X, WB_Y, WB_BAR, sig_mu_vld, WB_VLD)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            sig_change_val <= '0';
         elsif (WB_VLD = '1' and (reg_X /= WB_X or reg_Y /= WB_Y)) then -- reg_BAR /= WB_BAR) then
            sig_change_val <= '1';
         elsif (sig_mu_vld = '1') then
            sig_change_val <= '0';
         end if;
      end if;
   end process;
   
   
   -- write to input registers A,B,C
   input_registers : process (CLK, RST, WB_X, WB_Y, WB_BAR, WB_VLD)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            --reg_X <= (others => '0');
            --reg_Y <= (others => '0');
            --reg_BAR <= (others => '0');
         elsif (WB_VLD = '1') then
            reg_X <= WB_X;
            reg_Y <= WB_Y;
            reg_BAR <= WB_BAR;
         end if;
      end if;
   end process;
   
   MU_X <= reg_X;
   MU_Y <= reg_Y;
   MU_BAR <= reg_BAR(3 downto 0);
   
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic : process (state_reg, WB_VLD, MU_VLD, sig_change_val, RDY)
   begin
     state_next <= state_reg;  
     
     case state_reg is
        when init_state =>
		  if (RDY = '1') then
			 state_next <= wait_state;--get_value_state;
			 WB_ACT <= '0';
			 MU_ACT <= '0';  
	      else
			 state_next <= init_state;
			 WB_ACT <= '0';
			 MU_ACT <= '0'; 	
          end if;

		when wait_state =>
	      state_next <= get_value_state;
		  WB_ACT <= '0';
		  MU_ACT <= '0'; 	
        
        when get_value_state => 
          if (WB_VLD = '1') then 
             state_next <= save_state;
             WB_ACT <= '1';
             MU_ACT <= '0';
          else 
             state_next <= get_value_state;
             WB_ACT <= '1';
             MU_ACT <= '0';
          end if;
          
        when save_state => 
          if (sig_change_val = '0') then
             state_next <= init_state;
             WB_ACT <= '0';
             MU_ACT <= '0';
          elsif (MU_VLD = '1') then 
             state_next <= init_state;
             WB_ACT <= '0';
             MU_ACT <= '1';
          else 
             state_next <= save_state;
             WB_ACT <= '0';
             MU_ACT <= '1';
          end if;
      end case;      
   end process;  
end architecture;
