-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    add.vhd
-- Description:  Adder unit
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.9.2012 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- use work.math_pack.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity ADD is
   generic
   (
      DATA_WIDTH  : integer := 16       
   );

   port
   (
      --CLK         : in  std_logic;
      --RST         : in  std_logic;
      START       : in  std_logic;
            
      -- ----------------- INPUT INTERFACE ----------------------------------
      INPUT_A     : in  std_logic_vector(2*DATA_WIDTH-1 downto 0); -- register A
      INPUT_B     : in  std_logic_vector(2*DATA_WIDTH-1 downto 0); -- register B  
           
      -- ----------------- OUTPUT INTERFACE ---------------------------------      
      ADD_RESULT : out std_logic_vector(2*DATA_WIDTH-1 downto 0);
      ADD_VLD    : out std_logic
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture ADD_ARCH of ADD is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================


begin
   process (START, INPUT_A, INPUT_B)
   begin
      if (START = '1') then
         ADD_RESULT <= INPUT_A+INPUT_B;
         ADD_VLD <= '1';
      else
         ADD_RESULT <= (others=>'0');
         ADD_VLD <= '0';
      end if;
   end process;
end architecture;
