-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    peu_fsm_tl.vhd
-- Description:  Top level of peu and peu_fsm for testing funcionallity
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.1.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
--use ieee.numeric_std.all;
--use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity PEU_FSM_TL_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic; 
      
      -- ----------------- INPUT DATA --------------------------------- 
      A              : in std_logic_vector(DATA_WIDTH-1 downto 0);
      B              : in std_logic_vector(DATA_WIDTH-1 downto 0);
      C              : in std_logic_vector(DATA_WIDTH-1 downto 0);

      NOT_CYC_WRITE  : in std_logic;
              
      -- ----------------- OUTPUT DATA ---------------------------
      X              : out std_logic_vector(DATA_WIDTH-1 downto 0);
      Y              : out std_logic_vector(DATA_WIDTH-1 downto 0)
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture PEU_FSM_TL_ARCH of PEU_FSM_TL_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================

-- data signals
signal sig_A                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_B                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_C                  : std_logic_vector(DATA_WIDTH-1 downto 0);

signal sig_X                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_Y                  : std_logic_vector(DATA_WIDTH-1 downto 0);

-- control signals
signal sig_act                : std_logic;
signal sig_vld                : std_logic;


begin
   
   peu : entity work.peu_ent
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      CLK            => CLK,
      RST            => RST,
      
      -- input interface
      DIST_A         => sig_A, 
      DIST_B         => sig_B, 
      DIST_C         => sig_C, 
           
      -- output interface      
      X              => sig_X,
      Y              => sig_Y,
      
      -- control interface
      ACT            => sig_act,
      VLD            => sig_vld

   );
   
   peu_fsm : entity work.peu_fsm_ent 
   generic map (
      DATA_WIDTH  => DATA_WIDTH      
   )
   port map(
      CLK            => CLK,
      RST            => RST, 
      
      -- ----------------- INPUT/OUTPUT DATA --------------------- 
      A              => A,
      B              => B,
      C              => C,

      NOT_CYC_WRITE  => NOT_CYC_WRITE,

      X              => X,
      Y              => Y,
      
      -- ----------------- PEU DATA ---------------------------
      X_PEU          => sig_X,
      Y_PEU          => sig_Y,
      
      A_PEU          => sig_A, 
      B_PEU          => sig_B, 
      C_PEU          => sig_C, 
      
      ACT_PEU        => sig_act,
      VLD_PEU        => sig_vld
 
   ); 

end architecture;
