library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--use ieee.std_logic_unsigned.all;
use ieee.std_logic_signed.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 30 ns;
  constant reset_time   : time := 100 ns;
  constant data_width   : integer := 16;
  
  -- prepajacie signaly
  signal clk            : std_logic;
  signal rst            : std_logic;
      
      -- ----------------- input interface ----------------------- 
  signal a_in           : std_logic_vector(data_width-1 downto 0);
  signal b_in           : std_logic_vector(data_width-1 downto 0);
  signal c_in           : std_logic_vector(data_width-1 downto 0);
  signal s_0_in         : std_logic_vector(data_width-1 downto 0);
  signal s_1_in         : std_logic_vector(data_width-1 downto 0);
  signal s_2_in         : std_logic_vector(data_width-1 downto 0);
  signal s_3_in         : std_logic_vector(data_width-1 downto 0);
  signal data_vld       :  std_logic; 
              
      -- ----------------- output interface ----------------------
  signal a_out          : std_logic_vector(data_width-1 downto 0);
  signal b_out          : std_logic_vector(data_width-1 downto 0);
  signal c_out          : std_logic_vector(data_width-1 downto 0);
  signal s_0_out        : std_logic_vector(data_width-1 downto 0);
  signal s_1_out        : std_logic_vector(data_width-1 downto 0);
  signal s_2_out        : std_logic_vector(data_width-1 downto 0);
  signal s_3_out        : std_logic_vector(data_width-1 downto 0);
  signal act            : std_logic;
  signal vld            : std_logic;
  signal x_speed        : std_logic_vector(data_width-1 downto 0);
  signal y_speed        : std_logic_vector(data_width-1 downto 0);
  signal x_speed_in     : std_logic_vector(data_width-1 downto 0);
  signal y_speed_in     : std_logic_vector(data_width-1 downto 0);
      

begin
  -- instancia testovanej komponenty
  UUT : entity work.SIF_FSM_ENT
  port map( CLK            => clk,
            RST            => rst,
            -- ----------------- INPUT INTERFACE ----------------------- 
            A_IN           => a_in,
            B_IN           => b_in,
            C_IN           => c_in,
            S_0_IN         => s_0_in,
            S_1_IN         => s_1_in,
            S_2_IN         => s_2_in,
            S_3_IN         => s_3_in,
            DATA_VLD       => data_vld,
                    
            -- ----------------- OUTPUT INTERFACE ----------------------
            A_OUT          => a_out,
            B_OUT          => b_out,
            C_OUT          => c_out,
            S_0_OUT        => s_0_out,
            S_1_OUT        => s_1_out,
            S_2_OUT        => s_2_out,
            S_3_OUT        => s_3_out,
            ACT            => act,
            VLD            => vld,
            X_SPEED        => x_speed,
            Y_SPEED        => y_speed,    
            X_SPEED_IN     => x_speed_in,
            Y_SPEED_IN     => y_speed_in   
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clk     <='1';
    wait for clk_period/2;
    clk     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      rst   <= '1';
      wait for 2*reset_time;
      rst   <= '0';
      wait for 20*clk_period;
      rst   <= '0';
      wait;
   end process;

  -- generovanie vstupov
  behaviour : process
  begin
    vld <= '0';
    a_in <= std_logic_vector(to_unsigned(56, DATA_WIDTH));
    b_in <= std_logic_vector(to_unsigned(56, DATA_WIDTH));
    c_in <= std_logic_vector(to_unsigned(56, DATA_WIDTH));
    s_0_in <= std_logic_vector(to_unsigned(4, DATA_WIDTH));
    s_1_in <= std_logic_vector(to_unsigned(4, DATA_WIDTH));
    s_2_in <= std_logic_vector(to_unsigned(4, DATA_WIDTH));
    s_3_in <= std_logic_vector(to_unsigned(12, DATA_WIDTH));
    data_vld <= '1';
    
    wait for 2*reset_time+reset_time/2;
    wait for 5*clk_period;
    data_vld <= '0';
    a_in <= std_logic_vector(to_unsigned(64, DATA_WIDTH));
    b_in <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    c_in <= std_logic_vector(to_unsigned(37, DATA_WIDTH));
    s_0_in <= std_logic_vector(to_unsigned(4, DATA_WIDTH));
    s_1_in <= std_logic_vector(to_unsigned(28, DATA_WIDTH));
    s_2_in <= std_logic_vector(to_unsigned(12, DATA_WIDTH));
    s_3_in <= std_logic_vector(to_unsigned(4, DATA_WIDTH));
    data_vld <= '1';
    
    wait for reset_time/2;
    vld <= '1';
    wait for clk_period;
    vld <= '0';
    
    wait for reset_time;
    a_in <= std_logic_vector(to_unsigned(64, DATA_WIDTH));
    b_in <= std_logic_vector(to_unsigned(23, DATA_WIDTH));
    c_in <= std_logic_vector(to_unsigned(37, DATA_WIDTH));
    s_0_in <= std_logic_vector(to_unsigned(5, DATA_WIDTH));
    s_1_in <= std_logic_vector(to_unsigned(29, DATA_WIDTH));
    s_2_in <= std_logic_vector(to_unsigned(13, DATA_WIDTH));
    s_3_in <= std_logic_vector(to_unsigned(5, DATA_WIDTH));
    
    wait for reset_time;
    vld <= '1';
    wait for clk_period;
    vld <= '0';
    
    
    wait for 3*reset_time;
    vld <= '1';
    wait for clk_period;
    vld <= '0';
    
    
    wait;
  end process behaviour; 
  
  behaviour1 : process
  begin   
  x_speed_in <= std_logic_vector(to_unsigned(0, DATA_WIDTH));
  y_speed_in <= std_logic_vector(to_unsigned(0, DATA_WIDTH));
  wait for 10*clk_period;
  x_speed_in <= std_logic_vector(to_unsigned(8, DATA_WIDTH));
  y_speed_in <= std_logic_vector(to_unsigned(0, DATA_WIDTH));
  wait for 4* clk_period;
  x_speed_in <= std_logic_vector(to_unsigned(0, DATA_WIDTH));
  y_speed_in <= std_logic_vector(to_unsigned(0, DATA_WIDTH));
   wait for 10*clk_period;
  x_speed_in <= std_logic_vector(to_unsigned(0, DATA_WIDTH));
  y_speed_in <= std_logic_vector(to_unsigned(8, DATA_WIDTH));
  wait for 4* clk_period;
  x_speed_in <= std_logic_vector(to_unsigned(0, DATA_WIDTH));
  y_speed_in <= std_logic_vector(to_unsigned(0, DATA_WIDTH));
  
    
    wait;
  end process behaviour1; 
  
end architecture tb_arch;
