-- Generated by PERL program wishbone.pl. Do not edit this file.
--
-- For defines see wishbone.defines
--
-- Generated Mon Mar  4 11:56:48 2013
--
-- Wishbone masters:
--   sif
--   mu
--   pfu
--
-- Wishbone slaves:
--   peu
--     baseadr 0x10 - size 0x10
--   bdu
--     baseadr 0x20 - size 0x10
-----------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

package intercon_package is


function "and" (
  l : std_logic_vector;
  r : std_logic)
return std_logic_vector;
end intercon_package;
package body intercon_package is

function "and" (
  l : std_logic_vector;
  r : std_logic)
return std_logic_vector is
  variable result : std_logic_vector(l'range);
begin  -- "and"
  for i in l'range loop
  result(i) := l(i) and r;
end loop;  -- i
return result;
end "and";
end intercon_package;
library IEEE;
use IEEE.std_logic_1164.all;

entity trafic_supervision is

  generic (
    priority     : integer := 1;
    tot_priority : integer := 2);

  port (
    bg           : in  std_logic;       -- bus grant
    ce           : in  std_logic;       -- clock enable
    trafic_limit : out std_logic;
    clk          : in  std_logic;
    reset        : in  std_logic);

end trafic_supervision;

architecture rtl of trafic_supervision is

  signal shreg : std_logic_vector(tot_priority-1 downto 0);
  signal cntr : integer range 0 to tot_priority;

begin  -- rtl

  -- purpose: holds information of usage of latest cycles
  -- type   : sequential
  -- inputs : clk, reset, ce, bg
  -- outputs: shreg('left)
  sh_reg: process (clk,reset)
  begin  -- process shreg
    if reset = '1' then                 -- asynchronous reset (active hi)
      shreg <= (others=>'0');
    elsif clk'event and clk = '1' then  -- rising clock edge
      if ce='1' then
        shreg <= shreg(tot_priority-2 downto 0) & bg;
      end if;
    end if;
  end process sh_reg;

  -- purpose: keeps track of used cycles
  -- type   : sequential
  -- inputs : clk, reset, shreg('left), bg, ce
  -- outputs: trafic_limit
  counter: process (clk, reset)
  begin  -- process counter
    if reset = '1' then                 -- asynchronous reset (active hi)
      cntr <= 0;
      trafic_limit <= '0';
    elsif clk'event and clk = '1' then  -- rising clock edge
      if ce='1' then
        if bg='1' and shreg(tot_priority-1)='0' then
          cntr <= cntr + 1;
          if cntr=priority-1 then
            trafic_limit <= '1';
          end if;
        elsif bg='0' and shreg(tot_priority-1)='1' then
          cntr <= cntr - 1;
          if cntr=priority then
            trafic_limit <= '0';
          end if;
        end if;
      end if;
    end if;
  end process counter;

end rtl;

library IEEE;
use IEEE.std_logic_1164.all;
use work.intercon_package.all;

entity intercon is
  port (
  -- wishbone master port(s)
  -- sif
  sif_dat_i : out std_logic_vector(15 downto 0);
  sif_ack_i : out std_logic;
  sif_dat_o : in  std_logic_vector(15 downto 0);
  sif_we_o  : in  std_logic;
  sif_sel_o : in  std_logic_vector(1 downto 0);
  sif_adr_o : in  std_logic_vector(7 downto 0);
  sif_cyc_o : in  std_logic;
  sif_stb_o : in  std_logic;
  -- mu
  mu_dat_i : out std_logic_vector(15 downto 0);
  mu_ack_i : out std_logic;
  mu_dat_o : in  std_logic_vector(15 downto 0);
  mu_we_o  : in  std_logic;
  mu_sel_o : in  std_logic_vector(1 downto 0);
  mu_adr_o : in  std_logic_vector(7 downto 0);
  mu_cyc_o : in  std_logic;
  mu_stb_o : in  std_logic;
  -- pfu
  pfu_dat_i : out std_logic_vector(15 downto 0);
  pfu_ack_i : out std_logic;
  pfu_dat_o : in  std_logic_vector(15 downto 0);
  pfu_we_o  : in  std_logic;
  pfu_sel_o : in  std_logic_vector(1 downto 0);
  pfu_adr_o : in  std_logic_vector(7 downto 0);
  pfu_cyc_o : in  std_logic;
  pfu_stb_o : in  std_logic;
  -- wishbone slave port(s)
  -- peu
  peu_dat_o : in  std_logic_vector(15 downto 0);
  peu_ack_o : in  std_logic;
  peu_dat_i : out std_logic_vector(15 downto 0);
  peu_we_i  : out std_logic;
  peu_sel_i : out std_logic_vector(1 downto 0);
  peu_adr_i : out std_logic_vector(3 downto 0);
  peu_cyc_i : out std_logic;
  peu_stb_i : out std_logic;
  -- bdu
  bdu_dat_o : in  std_logic_vector(15 downto 0);
  bdu_ack_o : in  std_logic;
  bdu_dat_i : out std_logic_vector(15 downto 0);
  bdu_we_i  : out std_logic;
  bdu_sel_i : out std_logic_vector(1 downto 0);
  bdu_adr_i : out std_logic_vector(3 downto 0);
  bdu_cyc_i : out std_logic;
  bdu_stb_i : out std_logic;
  -- clock and reset
  clk   : in std_logic;
  reset : in std_logic);
end intercon;
architecture rtl of intercon is
  signal sif_bg : std_logic; -- bus grant
  signal mu_bg : std_logic; -- bus grant
  signal pfu_bg : std_logic; -- bus grant
  signal peu_ss : std_logic; -- slave select
  signal bdu_ss : std_logic; -- slave select
begin  -- rtl
arbiter_sharedbus: block
  signal sif_bg_1, sif_bg_2, sif_bg_q : std_logic;
  signal mu_bg_1, mu_bg_2, mu_bg_q : std_logic;
  signal pfu_bg_1, pfu_bg_2, pfu_bg_q : std_logic;
  signal sif_trafic_ctrl_limit : std_logic;
  signal mu_trafic_ctrl_limit : std_logic;
  signal pfu_trafic_ctrl_limit : std_logic;
  signal ack, ce, idle :std_logic;
begin -- arbiter
ack <= peu_ack_o or bdu_ack_o;

trafic_supervision_1 : entity work.trafic_supervision
generic map(
  priority => 7,
  tot_priority => 12)
port map(
  bg => sif_bg,
  ce => ce,
  trafic_limit => sif_trafic_ctrl_limit,
  clk => clk,
  reset => reset);

trafic_supervision_2 : entity work.trafic_supervision
generic map(
  priority => 3,
  tot_priority => 12)
port map(
  bg => mu_bg,
  ce => ce,
  trafic_limit => mu_trafic_ctrl_limit,
  clk => clk,
  reset => reset);

trafic_supervision_3 : entity work.trafic_supervision
generic map(
  priority => 2,
  tot_priority => 12)
port map(
  bg => pfu_bg,
  ce => ce,
  trafic_limit => pfu_trafic_ctrl_limit,
  clk => clk,
  reset => reset);

process(clk,reset)
begin
if reset='1' then
  sif_bg_q <= '0';
elsif clk'event and clk='1' then
if sif_bg_q='0' then
  sif_bg_q <= sif_bg;
elsif ack='1' then
  sif_bg_q <= '0';
end if;
end if;
end process;

process(clk,reset)
begin
if reset='1' then
  mu_bg_q <= '0';
elsif clk'event and clk='1' then
if mu_bg_q='0' then
  mu_bg_q <= mu_bg;
elsif ack='1' then
  mu_bg_q <= '0';
end if;
end if;
end process;

process(clk,reset)
begin
if reset='1' then
  pfu_bg_q <= '0';
elsif clk'event and clk='1' then
if pfu_bg_q='0' then
  pfu_bg_q <= pfu_bg;
elsif ack='1' then
  pfu_bg_q <= '0';
end if;
end if;
end process;

idle <= '1' when sif_bg_q='0' and mu_bg_q='0' and pfu_bg_q='0' else '0';
sif_bg_1 <= '1' when idle='1' and sif_cyc_o='1' and sif_trafic_ctrl_limit='0' else '0';
mu_bg_1 <= '1' when idle='1' and mu_cyc_o='1' and mu_trafic_ctrl_limit='0' and (sif_bg_1='0') else '0';
pfu_bg_1 <= '1' when idle='1' and pfu_cyc_o='1' and pfu_trafic_ctrl_limit='0' and (sif_bg_1='0' and mu_bg_1='0') else '0';
sif_bg_2 <= '1' when idle='1' and (sif_bg_1='0' and mu_bg_1='0' and pfu_bg_1='0') and sif_cyc_o='1' else '0';
mu_bg_2 <= '1' when idle='1' and (sif_bg_1='0' and mu_bg_1='0' and pfu_bg_1='0' and sif_bg_2='0') and mu_cyc_o='1' else '0';
pfu_bg_2 <= '1' when idle='1' and (sif_bg_1='0' and mu_bg_1='0' and pfu_bg_1='0' and sif_bg_2='0' and mu_bg_2='0') and pfu_cyc_o='1' else '0';
sif_bg <= sif_bg_q or sif_bg_1 or sif_bg_2;
mu_bg <= mu_bg_q or mu_bg_1 or mu_bg_2;
pfu_bg <= pfu_bg_q or pfu_bg_1 or pfu_bg_2;
ce <= sif_cyc_o or mu_cyc_o or pfu_cyc_o when idle='1' else '0';

end block arbiter_sharedbus;

decoder:block
  signal adr : std_logic_vector(7 downto 0);
begin
adr <= (sif_adr_o and sif_bg) or (mu_adr_o and mu_bg) or (pfu_adr_o and pfu_bg);
peu_ss <= '1' when adr(7 downto 4)="0001" else
'0';
bdu_ss <= '1' when adr(7 downto 4)="0010" else
'0';
peu_adr_i <= adr(3 downto 0);
bdu_adr_i <= adr(3 downto 0);
end block decoder;

mux: block
  signal cyc, stb, we, ack : std_logic;
  signal sel : std_logic_vector(1 downto 0);
  signal dat_m2s, dat_s2m : std_logic_vector(15 downto 0);
begin
cyc <= (sif_cyc_o and sif_bg) or (mu_cyc_o and mu_bg) or (pfu_cyc_o and pfu_bg);
peu_cyc_i <= peu_ss and cyc;
bdu_cyc_i <= bdu_ss and cyc;
stb <= (sif_stb_o and sif_bg) or (mu_stb_o and mu_bg) or (pfu_stb_o and pfu_bg);
peu_stb_i <= peu_ss and stb;
bdu_stb_i <= bdu_ss and stb;
we <= (sif_we_o and sif_bg) or (mu_we_o and mu_bg) or (pfu_we_o and pfu_bg);
peu_we_i <= peu_ss and we;
bdu_we_i <= bdu_ss and we;
ack <= peu_ack_o or bdu_ack_o;
sif_ack_i <= ack and sif_bg;
mu_ack_i <= ack and mu_bg;
pfu_ack_i <= ack and pfu_bg;
sel <= (sif_sel_o and sif_bg) or (mu_sel_o and mu_bg) or (pfu_sel_o and pfu_bg);
peu_sel_i <= sel;
bdu_sel_i <= sel;
dat_m2s <= (sif_dat_o and sif_bg) or (mu_dat_o and mu_bg) or (pfu_dat_o and pfu_bg);
peu_dat_i <= dat_m2s;
bdu_dat_i <= dat_m2s;
dat_s2m <= (peu_dat_o and peu_ss) or (bdu_dat_o and bdu_ss);
sif_dat_i <= dat_s2m;
mu_dat_i <= dat_s2m;
pfu_dat_i <= dat_s2m;
end block mux;

end rtl;