library ieee;
use ieee.std_logic_1164.all;

entity testbench is
end entity testbench;

architecture tb_arch of testbench is
  -- perioda hodin 
  constant clk_period   : time := 20 ns;
  constant reset_time   : time := 100 ns;
  
  signal clk               : std_logic;
  signal rst               : std_logic;
  
        -- SIF_WRITE_MASTER
  signal sif_act           : std_logic;
  signal sif_vld           : std_logic;
  signal sif_a             : std_logic_vector( 15 downto 0 );
  signal sif_b             : std_logic_vector( 15 downto 0 );
  signal sif_c             : std_logic_vector( 15 downto 0 );
  signal sif_s_0           : std_logic_vector( 15 downto 0);
  signal sif_s_1           : std_logic_vector( 15 downto 0);
  signal sif_s_2           : std_logic_vector( 15 downto 0);
  signal sif_s_3           : std_logic_vector( 15 downto 0);
      
      -- MU_READ_MASTER
  signal mu_act            : std_logic;
  signal mu_vld            : std_logic;
  signal mu_x              : std_logic_vector( 15 downto 0 );
  signal mu_y              : std_logic_vector( 15 downto 0 );
  signal mu_bar            : std_logic_vector( 15 downto 0 );
  
  -- PFU_READ_MASTER
  signal pfu_act           : std_logic;
  signal pfu_vld           : std_logic;
  signal pfu_x             : std_logic_vector( 15 downto 0 );
  signal pfu_y             : std_logic_vector( 15 downto 0 );
      
      -- PEU_SLAVE
  signal peu_a             : std_logic_vector( 15 downto 0 );
  signal peu_b             : std_logic_vector( 15 downto 0 );
  signal peu_c             : std_logic_vector( 15 downto 0 );
  signal peu_x             : std_logic_vector( 15 downto 0 );
  signal peu_y             : std_logic_vector( 15 downto 0 );
  signal peu_not_cyc_write : std_logic;
  signal peu_not_cyc_read  : std_logic;   

      -- BDU_SLAVE
  signal bdu_s_0            : std_logic_vector( 15 downto 0 );
  signal bdu_s_1            : std_logic_vector( 15 downto 0 );
  signal bdu_s_2            : std_logic_vector( 15 downto 0 );
  signal bdu_s_3            : std_logic_vector( 15 downto 0 );
  signal bdu_bar            : std_logic_vector( 15 downto 0 );
  signal bdu_not_cyc_write  : std_logic;
  signal bdu_not_cyc_read   : std_logic;

begin
  -- instancia testovanej komponenty
  UUT : entity work.top_level
  port map( 
      CLK               => clk,
      RST               => rst,
      -- SIF_WRITE_MASTER
      SIF_ACT           => sif_act,
      SIF_VLD           => sif_vld,
      SIF_A             => sif_a,
      SIF_B             => sif_b,
      SIF_C             => sif_c,
      SIF_S_0           => sif_s_0,
      SIF_S_1           => sif_s_1,
      SIF_S_2           => sif_s_2,
      SIF_S_3           => sif_s_3,
      
      -- MU_READ_MASTER
      MU_ACT            => mu_act,
      MU_VLD            => mu_vld,
      MU_X              => mu_x,
      MU_Y              => mu_y,
      MU_BAR            => mu_bar,
      
      -- PFU_READ_MASTER
      PFU_ACT           => pfu_act,
      PFU_VLD           => pfu_vld, 
      PFU_X             => pfu_x,
      PFU_Y             => pfu_y,
      
      -- PEU_SLAVE
      PEU_A             => peu_a,
      PEU_B             => peu_b,
      PEU_C             => peu_c,
      PEU_X             => peu_x,
      PEU_Y             => peu_y,
      PEU_NOT_CYC_WRITE => peu_not_cyc_write,
      PEU_NOT_CYC_READ  => peu_not_cyc_read,
      
      -- BDU_SLAVE
      BDU_S_0           => bdu_s_0,
      BDU_S_1           => bdu_s_1,
      BDU_S_2           => bdu_s_2,
      BDU_S_3           => bdu_s_3,
      BDU_BAR           => bdu_bar,
      BDU_NOT_CYC_WRITE => bdu_not_cyc_write,
      BDU_NOT_CYC_READ  => bdu_not_cyc_read
      
  );

  -- generovanie hodinoveho signalu 
  clk_gen :  process 
  begin  
    clk     <='1';
    wait for clk_period/2;
    clk     <='0';
    wait for clk_period/2;
  end process clk_gen;
  
  resetgen: process
   begin
      rst   <= '1';
      wait for reset_time;
      rst   <= '0';
      
      wait for 2*reset_time;
      --rst_i   <= '1';
      wait for clk_period;
      rst   <= '0';
      wait;
   end process;

  -- generovanie vstupov
  behaviour : process
  begin
    sif_act <= '0'; 
    sif_a <= "1010101010101010";
    sif_b <= "1111000011110000";
    sif_c <= "1100110011001100";
    sif_s_0 <= "1111001001111000";
    sif_s_1 <= "1111000011100010";
    sif_s_2 <= "1011100001111000";
    sif_s_3 <= "1111000011011000";
    wait for reset_time;
    wait for clk_period;
    sif_act <= '1'; 
    wait for 22*clk_period;
    sif_act <= '0'; 
    sif_a <= "0101010101010101";
    sif_b <= "0111100001111000";
    sif_c <= "0110011001100110";
    wait for 30*clk_period;
    sif_act <= '1'; 
    wait for 22*clk_period;
    sif_act <= '0'; 
    wait;
  end process behaviour; 
  
    -- generovanie vstupov
  behaviour1 : process
  begin
    mu_act <= '0';  
    peu_x <= "0101010101010101";
    peu_y <= "0111100001111000";
    bdu_bar <= "0001110001110001";
    wait for reset_time;
    wait for 25*clk_period;
    mu_act <= '1';
    wait for 10*clk_period;
    mu_act <= '0';
    wait;
  end process behaviour1;
  
      -- generovanie vstupov
  behaviour2 : process
  begin
    pfu_act <= '0';  
    wait for reset_time;
    wait for 40*clk_period;
    pfu_act <= '1';
    wait for 7*clk_period;
    pfu_act <= '0';
    wait;
  end process behaviour2;
   
end architecture tb_arch;
